/***************************************************************************
                          filters.cxx  -  description
                             -------------------
    begin                : Fri Jun 30 2000
    copyright            : (C) 2000 by Hans Dijkema
    email                : kmailcvt@hum.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "filters.hxx"
#include "kmailcvt.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

filterInfo::filterInfo(QWidget *parent,char *name)
{
int w,h;
   _parent=parent;

   w=parent->width();
   h=parent->height();
   w-=20;

   h-=100;
   _log=new QListBox(parent);
   _log->setGeometry(10,h,w,90);

   h-=30;
   _done_overall=new QProgressBar(100,parent);
   _done_overall->setGeometry(10,h,w,20);
   h-=30;
   _done_current=new QProgressBar(100,parent);
   _done_current->setGeometry(10,h,w,20);

   _current=new QLabel(parent);
   _current->setText("current:");
   h-=(_current->height()+2);
   _current->setGeometry(10,h,w,_current->height());
   _to=new QLabel(parent);
   _to->setText("to:");
   h-=(_to->height()+2);
   _to->setGeometry(10,h,w,_to->height());
   _from=new QLabel(parent);
//   _from->setFrameStyle(QFrame::Panel|QFrame::Sunken);
//   _from->setLineWidth(2);
   _from->setText("from:");
   h-=(_from->height()+1);
   _from->setGeometry(10,h,w,_from->height());
}

filterInfo::~filterInfo()
{
  delete _log;
  delete _from;
  delete _to;
  delete _current;
  delete _done_overall;
  delete _done_current;
}

void  filterInfo::from(const char *from)
{
  _from->setText(from);
}

void  filterInfo::to(const char *to)
{
  _to->setText(to);
}

void  filterInfo::current(const char *current)
{
  _current->setText(current);
}

void  filterInfo::current(float percent)
{
int p=(int) (percent+0.5);
  if (percent<0) { _done_current->reset(); }
  _done_current->setProgress(p);
  procEvents();
}

void  filterInfo::overall(float percent)
{
int p=(int) (percent+0.5);
  if (percent<0) { _done_overall->reset(); }
  _done_overall->setProgress(p);
}

void filterInfo::log(const char *toLog)
{
  _log->insertItem(toLog);
  _log->setCurrentItem(_log->count()-1);
  _log->centerCurrentItem();
  procEvents();
//  fprintf(stderr,"%s\n",toLog);
}

void filterInfo::clear(void)
{
  _log->clear();
  current();
  overall();
  current("");
  from("");
  to("");
}

void  filterInfo::alert(const char *conversion,const char *message)
{
  QMessageBox::information(_parent,conversion,message);
}

void filterInfo::adjWidth(QWidget *q)
{
int w=_parent->width()-20;
  q->resize(w,q->height());
}

void filterInfo::adjustSize(void)
{
  adjWidth(_log);
  adjWidth(_from);
  adjWidth(_to);
  adjWidth(_current);
  adjWidth(_done_overall);
  adjWidth(_done_current);
}

filter::filter(char *_name)
{
  myName=strdup(_name);
}

filter::~filter()
{
  delete myName;
}

char *filter::name(void)
{
return myName;
}

void filter::import(filterInfo *info,QWidget *parent)
{
  info->alert("class filter","no import function implemented");
}



filters::filters(filterInfo *i,QWidget *_parent,char *name) : QComboBox(_parent,name)
{
  info=i;
  parent=_parent;
}

filters::~filters()
{
int i;
  for(i=0;i<F.len();i++) {
    delete F[i];
  }
}

void filters::add(filter *f)
{
  F[F.len()]=f;
  insertItem(f->name());
}

void filters::import(void)
{
  info->clear();
  F[currentItem()]->import(info,parent);
}


kmail::kmail()
{}

kmail::~kmail()
{}

void  kmail::kmailMessage(filterInfo *info,char *folder,FILE *msg)
{
FILE *f;
char  FOLDER[1024];
int   fh,bytes;
char  buf[4096];
QWidget *parent=info->parent();

  sprintf(FOLDER,"%s/Mail",getenv("HOME"));
  mkdir(FOLDER,S_IRUSR|S_IWUSR|S_IXUSR);    // This makes $HOME/Mail if it doesn't exist
  sprintf(FOLDER,"%s/Mail/%s",getenv("HOME"),folder);
  f=fopen(FOLDER,"ab");
  fseek(msg,0,SEEK_SET);
  fh=fileno(msg);
  while((bytes=read(fh,buf,4096))>0) {
    fwrite(buf,bytes,1,f);
  }
  fclose(f);
}


kab::kab()
{
  api=widget;
}

kab::~kab()
{
  api=NULL;
}

bool kab::kabStart(filterInfo *info,QWidget *parent)
{
  //api=new KabAPI(parent);
  if (api!=NULL) { return true; }

  if (init(false)!=KabAPI::NoError) {
     info->alert("Kmailcvt - K Adress Book API","Close down Kab to import .PAB files!");
     api=NULL;
     return false;
  }
  else {
     api=widget;
     return true;
  }
}

void kab::kabStop(filterInfo *info,QWidget *parent)
{
//  delete api;
//  api=NULL;
}

void kab::address(filterInfo *info,QWidget *parent,
                      char *givenname, char *email,
                      char *title,char *firstName,char *additionalName,char *lastName,
                      char *adress,char *town,char *state,char *zip,char *country,
                      char *organization,char *department,char *subDep,char *job,
                      char *tel,char *fax,char *mobile,char *modem,
                      char *homepage,char *talk,
                      char *comment,char *birthday
                     )
{
AddressBook::Entry e,ee;
string key=givenname;
bool exists;
int i,N;

  // Update the info

//  info->alert("he","he");
  N=api->noOfEntries();
//  info->alert("he","he");
  exists=false;
  for(i=0,api->first();i<N && !exists;i++,api->next()) {
    api->currentEntry(e);
    if (e.fn==key) { exists=true; }
  }
  if (exists) { api->previous(); }
  else {
    e.fn="";
    e.name="";
    e.firstname="";
    e.namePrefix="";
    e.additionalName="";
    e.address="";
    e.town="";
    e.state="";
    e.zip="";
    e.country="";
    e.emails.clear();
    e.talk.clear();
    e.telephone="";
    e.fax="";
    e.modem="";
    e.URL="";
    e.org="";
    e.orgUnit="";
    e.orgSubUnit="";
    e.comment="";
    //e.birthday="";
  }

  e.fn=givenname;

  if (email!=NULL)          {
    string mail;
    list<string>::iterator em;

    mail=email;
    {unsigned int i;
      for(i=0;i<mail.length() && mail[i]==' ';i++);
      if (i==mail.length()) { mail=""; }
    }
    if (mail!="") {
      for(em=e.emails.begin();
            em!=e.emails.end()
            && *em!=mail;
          ++em);

      if (em==e.emails.end()) {
        e.emails.insert(e.emails.end(),mail);
      }
    }
  }

  if (lastName!=NULL)       { e.name=lastName; }
  if (firstName!=NULL)      { e.firstname=firstName; }
  if (title!=NULL)          { e.namePrefix=title; }
  if (additionalName!=NULL) { e.additionalName=additionalName; }

  if (adress!=NULL)         { e.address=adress; }
  if (town!=NULL)           { e.town=town; }
  if (state!=NULL)          { e.state=state; }
  if (zip!=NULL)            { e.zip=zip; }
  if (country!=NULL)        { e.country=country; }

  {string etel=e.telephone,emob;
   int i;

    if (etel[etel.length()-1]==')') {
      for(i=etel.length()-1;i>=0 && etel[i]!='(';i--);
      emob=etel.substr(i);
      if (i<=1) { etel=""; }
      else {
        while(etel[i]==' ') { i--; }
        etel=etel.substr(0,i);
      }
    }

    if (tel!=NULL) { etel=tel; }
    if (mobile!=NULL) { emob="(";emob+=mobile;emob+=")"; }

    e.telephone=etel+" "+emob;
  }

  if (fax!=NULL)            { e.fax=fax; }
  if (modem!=NULL)          { e.modem=modem; }
  if (homepage!=NULL)       { e.URL=homepage; }
 // if (talk!=NULL)           { string t=talk; e.talk.add(t); }

  if (organization!=NULL)   { e.org=organization; }
  if (department!=NULL)     { e.orgUnit=department; }
  if (subDep!=NULL)         { e.orgSubUnit=subDep; }

  if (comment!=NULL)        { e.comment=comment; }
  //if (birthday!=NULL)       { e.birthday=birthday; }

  // Add the entry

  if (!exists) { api->add(e,key); }
  else {
    api->change(e);
  }
}
